@@ -66,7 +66,7 @@ static int call_stream_address_gstring(GString *o, struct packet_stream *ps, enu
 	int len, ret;
 	char buf[64]; /* 64 bytes ought to be enough for anybody */
 
-	ret = call_stream_address46(buf, ps, format, &len, NULL, 1);
+	ret = call_stream_address46(buf, ps, format, &len, NULL, true);
 	g_string_append_len(o, buf, len);
 	return ret;
 }
@@ -126,7 +126,7 @@ static int addr_parse_udp(struct stream_params *sp, char **out) {
 
 	SP_SET(sp, SEND);
 	SP_SET(sp, RECV);
-	sp->protocol = &transport_protocols[PROTO_RTP_AVP];
+	sp->protocol = &transport_protocols[PROTO_UNKNOWN];
 
 	if (out[RE_UDP_UL_ADDR4] && *out[RE_UDP_UL_ADDR4]) {
 		if (sockaddr_parse_any(&sp->rtp_endpoint.address, out[RE_UDP_UL_ADDR4]))
@@ -279,7 +279,7 @@ static int streams_parse_func(char **a, void **ret, void *p) {
 
 	SP_SET(sp, SEND);
 	SP_SET(sp, RECV);
-	sp->protocol = &transport_protocols[PROTO_RTP_AVP];
+	sp->protocol = &transport_protocols[PROTO_UNKNOWN];
 
 	if (endpoint_parse_port_any(&sp->rtp_endpoint, a[0], atoi(a[1])))
 		goto fail;
@@ -314,6 +314,14 @@ void call_unlock_release(struct call **c) {
 	rwlock_unlock_w(&(*c)->master_lock);
 	obj_put(*c);
 }
+INLINE void call_unlock_release_update(struct call **c) {
+	if (!*c)
+		return;
+	rwlock_unlock_w(&(*c)->master_lock);
+	redis_update_onekey(*c, rtpe_redis_write);
+	obj_put(*c);
+	*c = NULL;
+}
 
 
 
@@ -358,12 +366,8 @@ static str *call_request_lookup_tcp(char **out, enum call_opmode opmode) {
 	ret = streams_print(&dialogue[1]->medias, 1, s.length, NULL, SAF_TCP);
 
 out2:
-	rwlock_unlock_w(&c->master_lock);
-
-	redis_update_onekey(c, rtpe_redis_write);
-
+	call_unlock_release_update(&c);
 	ilog(LOG_INFO, "Returning to SIP proxy: "STR_FORMAT"", STR_FMT0(ret));
-	obj_put(c);
 
 out:
 	g_hash_table_destroy(infohash);
@@ -1048,6 +1052,7 @@ static void call_ng_codec_flags(struct sdp_ng_flags *out, str *key, bencode_item
 #endif
 	ilog(LOG_WARN, "Unknown 'codec' operation encountered: '" STR_FORMAT "'", STR_FMT(key));
 }
+#ifdef WITH_TRANSCODING
 static void call_ng_parse_block_mode(str *s, enum block_dtmf_mode *output) {
 	switch (__csh_lookup(s)) {
 		case CSH_LOOKUP("off"):
@@ -1077,6 +1082,7 @@ static void call_ng_parse_block_mode(str *s, enum block_dtmf_mode *output) {
 					STR_FMT(s));
 	}
 }
+#endif
 static void call_ng_main_flags(struct sdp_ng_flags *out, str *key, bencode_item_t *value) {
 	str s = STR_NULL;
 	bencode_item_t *it;
@@ -2951,6 +2957,7 @@ const char *call_publish_ng(bencode_item_t *input, bencode_item_t *output, const
 	AUTO_CLEANUP(GQueue streams, sdp_streams_free) = G_QUEUE_INIT;
 	AUTO_CLEANUP(str sdp_in, str_free_dup) = STR_NULL;
 	AUTO_CLEANUP(str sdp_out, str_free_dup) = STR_NULL;
+	AUTO_CLEANUP_NULL(struct call *call, call_unlock_release);
 
 	call_ng_process_flags(&flags, input, OP_PUBLISH);
 
@@ -2968,7 +2975,7 @@ const char *call_publish_ng(bencode_item_t *input, bencode_item_t *output, const
 	if (sdp_streams(&parsed, &streams, &flags))
 		return "Incomplete SDP specification";
 
-	struct call *call = call_get_or_create(&flags.call_id, false, false);
+	call = call_get_or_create(&flags.call_id, false, false);
 	updated_created_from(call, addr, sin);
 	struct call_monologue *ml = call_get_or_create_monologue(call, &flags.from_tag);
 
@@ -2984,12 +2991,12 @@ const char *call_publish_ng(bencode_item_t *input, bencode_item_t *output, const
 		sdp_out = STR_NULL; // ownership passed to output
 	}
 
-	rwlock_unlock_w(&call->master_lock);
-	obj_put(call);
+	if (ret)
+		return "Failed to create SDP";
 
-	if (!ret)
-		return NULL;
-	return "Failed to create SDP";
+	call_unlock_release_update(&call);
+
+	return NULL;
 }
 
 
@@ -3113,6 +3120,8 @@ const char *call_subscribe_request_ng(bencode_item_t *input, bencode_item_t *out
 
 	bencode_dictionary_add_str_dup(output, "to-tag", &dest_ml->tag);
 
+	call_unlock_release_update(&call);
+
 	return NULL;
 }
 
@@ -3150,6 +3159,8 @@ const char *call_subscribe_answer_ng(bencode_item_t *input, bencode_item_t *outp
 	if (ret)
 		return "Failed to process subscription answer";
 
+	call_unlock_release_update(&call);
+
 	return NULL;
 }
 
@@ -3178,6 +3189,8 @@ const char *call_unsubscribe_ng(bencode_item_t *input, bencode_item_t *output) {
 	if (ret)
 		return "Failed to unsubscribe";
 
+	call_unlock_release_update(&call);
+
 	return NULL;
 }
 
