@@ -91,14 +91,10 @@ static int __k_null(struct rtpengine_srtp *s, struct packet_stream *);
 static int __k_srtp_encrypt(struct rtpengine_srtp *s, struct packet_stream *);
 static int __k_srtp_decrypt(struct rtpengine_srtp *s, struct packet_stream *);
 
-static int call_avp2savp_rtp(str *s, struct packet_stream *, struct stream_fd *, const endpoint_t *,
-		const struct timeval *, struct ssrc_ctx *);
-static int call_savp2avp_rtp(str *s, struct packet_stream *, struct stream_fd *, const endpoint_t *,
-		const struct timeval *, struct ssrc_ctx *);
-static int call_avp2savp_rtcp(str *s, struct packet_stream *, struct stream_fd *, const endpoint_t *,
-		const struct timeval *, struct ssrc_ctx *);
-static int call_savp2avp_rtcp(str *s, struct packet_stream *, struct stream_fd *, const endpoint_t *,
-		const struct timeval *, struct ssrc_ctx *);
+static int call_avp2savp_rtp(str *s, struct packet_stream *, struct ssrc_ctx *);
+static int call_savp2avp_rtp(str *s, struct packet_stream *, struct ssrc_ctx *);
+static int call_avp2savp_rtcp(str *s, struct packet_stream *, struct ssrc_ctx *);
+static int call_savp2avp_rtcp(str *s, struct packet_stream *, struct ssrc_ctx *);
 
 
 static struct logical_intf *__get_logical_interface(const str *name, sockfamily_t *fam);
@@ -189,6 +185,14 @@ const struct transport_protocol transport_protocols[] = {
 		.avpf		= 1,
 		.tcp		= 0,
 	},
+	[PROTO_UNKNOWN] = {
+		.index		= PROTO_UNKNOWN,
+		.name		= "unknown (legacy)",
+		.rtp		= 0,
+		.srtp		= 0,
+		.avpf		= 0,
+		.tcp		= 0,
+	},
 };
 const int num_transport_protocols = G_N_ELEMENTS(transport_protocols);
 
@@ -351,6 +355,7 @@ static const struct streamhandler * const __sh_matrix_noop[__PROTO_LAST] = { //
 	[PROTO_UDPTL]			= &__sh_noop,
 	[PROTO_RTP_SAVP_OSRTP]		= &__sh_noop,
 	[PROTO_RTP_SAVPF_OSRTP]		= &__sh_noop,
+	[PROTO_UNKNOWN]			= &__sh_noop,
 };
 
 /* ********** */
@@ -365,6 +370,7 @@ static const struct streamhandler * const * const __sh_matrix[__PROTO_LAST] = {
 	[PROTO_UDPTL]			= __sh_matrix_noop,
 	[PROTO_RTP_SAVP_OSRTP]		= __sh_matrix_in_rtp_savp,
 	[PROTO_RTP_SAVPF_OSRTP]		= __sh_matrix_in_rtp_savpf,
+	[PROTO_UNKNOWN]			= __sh_matrix_noop,
 };
 /* special case for DTLS as we can't pass through SRTP<>SRTP */
 static const struct streamhandler * const * const __sh_matrix_recrypt[__PROTO_LAST] = {
@@ -377,6 +383,7 @@ static const struct streamhandler * const * const __sh_matrix_recrypt[__PROTO_LA
 	[PROTO_UDPTL]			= __sh_matrix_noop,
 	[PROTO_RTP_SAVP_OSRTP]		= __sh_matrix_in_rtp_savp_recrypt,
 	[PROTO_RTP_SAVPF_OSRTP]		= __sh_matrix_in_rtp_savpf_recrypt,
+	[PROTO_UNKNOWN]			= __sh_matrix_noop,
 };
 
 /* ********** */
@@ -1065,23 +1072,19 @@ static int rtcp_demux(const str *s, struct call_media *media) {
 	return rtcp_demux_is_rtcp(s) ? 2 : 1;
 }
 
-static int call_avp2savp_rtp(str *s, struct packet_stream *stream, struct stream_fd *sfd, const endpoint_t *src,
-		const struct timeval *tv, struct ssrc_ctx *ssrc_ctx)
+static int call_avp2savp_rtp(str *s, struct packet_stream *stream, struct ssrc_ctx *ssrc_ctx)
 {
 	return rtp_avp2savp(s, &stream->crypto, ssrc_ctx);
 }
-static int call_avp2savp_rtcp(str *s, struct packet_stream *stream, struct stream_fd *sfd, const endpoint_t *src,
-		const struct timeval *tv, struct ssrc_ctx *ssrc_ctx)
+static int call_avp2savp_rtcp(str *s, struct packet_stream *stream, struct ssrc_ctx *ssrc_ctx)
 {
 	return rtcp_avp2savp(s, &stream->crypto, ssrc_ctx);
 }
-static int call_savp2avp_rtp(str *s, struct packet_stream *stream, struct stream_fd *sfd, const endpoint_t *src,
-		const struct timeval *tv, struct ssrc_ctx *ssrc_ctx)
+static int call_savp2avp_rtp(str *s, struct packet_stream *stream, struct ssrc_ctx *ssrc_ctx)
 {
 	return rtp_savp2avp(s, &stream->selected_sfd->crypto, ssrc_ctx);
 }
-static int call_savp2avp_rtcp(str *s, struct packet_stream *stream, struct stream_fd *sfd, const endpoint_t *src,
-		const struct timeval *tv, struct ssrc_ctx *ssrc_ctx)
+static int call_savp2avp_rtcp(str *s, struct packet_stream *stream, struct ssrc_ctx *ssrc_ctx)
 {
 	return rtcp_savp2avp(s, &stream->selected_sfd->crypto, ssrc_ctx);
 }
@@ -1944,7 +1947,7 @@ static int media_packet_decrypt(struct packet_handler_ctx *phc)
 	int ret = 0;
 	if (phc->decrypt_func) {
 		str ori_s = phc->s;
-		ret = phc->decrypt_func(&phc->s, phc->in_srtp, phc->mp.sfd, &phc->mp.fsin, &phc->mp.tv, phc->mp.ssrc_in);
+		ret = phc->decrypt_func(&phc->s, phc->in_srtp, phc->mp.ssrc_in);
 		// XXX for stripped auth tag and duplicate invocations of rtp_payload
 		// XXX transcoder uses phc->mp.payload
 		phc->mp.payload.len -= ori_s.len - phc->s.len;
@@ -1983,7 +1986,7 @@ int media_packet_encrypt(rewrite_func encrypt_func, struct packet_stream *out, s
 
 	for (GList *l = mp->packets_out.head; l; l = l->next) {
 		struct codec_packet *p = l->data;
-		int encret = encrypt_func(&p->s, out, NULL, NULL, NULL, mp->ssrc_out);
+		int encret = encrypt_func(&p->s, out, mp->ssrc_out);
 		if (encret == 1)
 			ret |= 0x02;
 		else if (encret != 0)
@@ -2164,11 +2167,22 @@ update_addr:
 	/* check the destination address of the received packet against what we think our
 	 * local interface to use is */
 	if (phc->mp.stream->selected_sfd && phc->mp.sfd != phc->mp.stream->selected_sfd) {
-		ilog(LOG_INFO, "Switching local interface to %s", endpoint_print_buf(&phc->mp.sfd->socket.local));
-		phc->mp.stream->selected_sfd = phc->mp.sfd;
-		phc->unkernelize = true;
-		phc->update = true;
-		phc->unkernelize_subscriptions = true;
+		// make sure the new interface/socket is actually one from the list of sockets
+		// that we intend to use, and not an old one from a previous negotiation
+		GList *contains = g_queue_find(&phc->mp.stream->sfds, phc->mp.sfd);
+		if (!contains)
+			ilog(LOG_INFO | LOG_FLAG_LIMIT, "Not switching from local socket %s to %s (not in list)",
+					endpoint_print_buf(&phc->mp.stream->selected_sfd->socket.local),
+					endpoint_print_buf(&phc->mp.sfd->socket.local));
+		else {
+			ilog(LOG_INFO | LOG_FLAG_LIMIT, "Switching local socket from %s to %s",
+					endpoint_print_buf(&phc->mp.stream->selected_sfd->socket.local),
+					endpoint_print_buf(&phc->mp.sfd->socket.local));
+			phc->mp.stream->selected_sfd = phc->mp.sfd;
+			phc->unkernelize = true;
+			phc->update = true;
+			phc->unkernelize_subscriptions = true;
+		}
 	}
 
 out:
